/*
 * Project: MoleCuilder
 * Description: creates and alters molecular systems
 * Copyright (C)  2010 University of Bonn. All rights reserved.
 * Please see the LICENSE file or "Copyright notice" in builder.cpp for details.
 */

/*
 * TxMenu.cpp
 *
 *  Created on: Dec 10, 2009
 *      Author: crueger
 */

// include config.h
#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "Helpers/MemDebug.hpp"

#include <boost/bind.hpp>
#include <algorithm>
#include <iostream>
#include <cmath>
#include "Actions/Action.hpp"
#include "Actions/ActionTraits.hpp"
#include "Menu/TextMenu/TxMenu.hpp"
#include "Menu/TextMenu/MenuItem.hpp"
#include "Helpers/Assert.hpp"


/**
 * produce a text menu with a given title.
 * The text will later be displayed using the stream passed to the constructor.
 */
TxMenu::TxMenu(std::ostream& _outputter, const std::string _title, char _spacer,int _length) :
  defaultItem(0),
  outputter(_outputter),
  title(_title),
  spacer(_spacer),
  length(_length),
  quit(false)
{
}

TxMenu::~TxMenu()
{
  for(std::list<MenuItem*>::iterator it=items.begin(); it != items.end(); it++)
    delete (*it);
}

void TxMenu::addItem(MenuItem* item) {
  items.push_back(item);
}

void TxMenu::removeItem(MenuItem* item) {
  items.remove(item);
}

void TxMenu::doQuit(){
  quit = true;
}

bool TxMenu::hasQuit(){
  return quit;
}

void TxMenu::showEntry(MenuItem* entry){
  if(entry->isActive()==false){
    outputter << "(";
  }
  outputter << entry->formatEntry();
  if(entry->isActive()==false){
    outputter << ")";
  }
  outputter << "\n";
}

void TxMenu::display() {
  char choice;
  bool somethingChosen = false;
  quit = false;
  do {
    int pre = floor((length - title.length()) /2.0);
    int post = ceil((length - title.length()) /2.0);
    for(int i=0;i<pre;i++)
      outputter << spacer;
    outputter << title;
    for(int i=0;i<post;i++)
          outputter << spacer;
    outputter << '\n';
    for_each(items.begin(), items.end(), boost::bind(&TxMenu::showEntry,this,_1));
    outputter.flush();

    std::cin >> choice;

    std::list<MenuItem*>::iterator iter;
    for(iter = items.begin(); iter!=items.end();iter++){
      if((*iter)->isActive()){
        somethingChosen |= (*iter)->checkTrigger(choice);
      }
    }
    // see if something was chosen and call default Item if not
    if(!somethingChosen) {
      if(defaultItem){
        defaultItem->doTrigger();
      }
      else{
        outputter << "Invalid Choice!" <<  std::endl;
      }
    }
  }while (!hasQuit());
}

std::string TxMenu::getTitle(){
  return title;
}

std::ostream& TxMenu::getOutputter()
{
  return outputter;
}


void TxMenu::addDefault(MenuItem* _defaultItem) {
  defaultItem = _defaultItem;
}

/****************************** Contained Actions ****************/

TxMenu::LeaveAction::LeaveAction(TxMenu* const _menu, const ActionTraits & LeaveActionTrait) :
  Action(LeaveActionTrait, true),
  menu(_menu)
{}

TxMenu::LeaveAction::~LeaveAction(){}

bool TxMenu::LeaveAction::canUndo(){
  return false;
}

bool TxMenu::LeaveAction::shouldUndo(){
  return false;
}

void TxMenu::LeaveAction::getParametersfromValueStorage()
{}

Dialog* TxMenu::LeaveAction::fillDialog(Dialog *dialog){
  ASSERT(dialog,"No Dialog given when filling action dialog");
  return dialog;
}


Action::state_ptr TxMenu::LeaveAction::performCall(){
  menu->doQuit();
  return Action::success;
}


Action::state_ptr TxMenu::LeaveAction::performUndo(Action::state_ptr){
  ASSERT(0,"Cannot undo leaving a menu");
  return Action::success;
}

Action::state_ptr TxMenu::LeaveAction::performRedo(Action::state_ptr){
  ASSERT(0,"Cannot redo leaving a menu");
  return Action::success;
}
